/*
 * ?? 2010 submission "Vanadium"
 * access pattern dump
 * Copyright (C) Jack Whitham 2010
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <unistd.h>

#include "tetra.h"
#include "smmu.h"
#include "cache.h"
#include "memlib.h"

#include "u64.hh"

#define UNDEFINED (~((ulongint) 0))

typedef struct Pattern_Data_struct {
    ulongint        event_time;
    ulongint        bus_latency;
    ulongint        bus_width;
} Pattern_Data;


static void Pattern_Txfer(Tetra_Params * tp,
                    ulongint size, ulongint min_cycles)
{
    Pattern_Data *  tdma_data = (Pattern_Data *) tp->user_cb;
    ulongint        start_time = tdma_data->event_time;
    ulongint        i, end_time, middle_time;

    assert (size > 0);
    assert (size < UINT_MAX);

    middle_time = start_time + tdma_data->bus_latency;
    end_time = middle_time +
        ((size + tdma_data->bus_width - 1) / tdma_data->bus_width);
    for (i = start_time; i < middle_time; i++) {
        printf("%04"FU" setup %"FU"\n", i, size);
    }
    for (i = middle_time; i < end_time; i++) {
        assert (size > 0);
        printf("%04"FU" burst %"FU"\n", i, size);
        if (size >= tdma_data->bus_width) {
            size -= tdma_data->bus_width;
        } else {
            size = 0;
        }
    }
    tdma_data->event_time = end_time;
}

static void Pattern_Tick (Tetra_Params * tp, Tick_Type tt)
{
    Pattern_Data *     tdma_data = (Pattern_Data *) tp->user_cb;

    switch (tt) {
        case TICK_LOOP_OVERHEAD:
            printf("%04"FU" loop\n", tdma_data->event_time);
            break;
        case TICK_ON_CHIP_ACCESS:
            printf("%04"FU"\n", tdma_data->event_time);
            break;
    }
    tdma_data->event_time ++;
}

static void Process_Method(ulongint number, ulongint bus_width,
                ulongint bus_latency, ulongint tile_size)
{
    Tetra_Params *  tp = Init_Params ();
    Pattern_Data    pd;

    printf("%s\n", tetra_descs[ number ]);
    memset(&pd, 0, sizeof(Pattern_Data));
    pd.bus_width = bus_width;
    pd.bus_latency = bus_latency;

    tp->user_cb = &pd;
    tp->smmu_very_deterministic = 1;
    tp->tiling_flag = 1;
    tp->tile_size = tile_size;

    Init_SMMU(tp, 16, CM_OPTION_B);
    tp->do_txfer_fn = Pattern_Txfer;
    tp->do_tick_fn = Pattern_Tick;
    tetra_methods[ number ](tp);
    tp->flush_fn(tp->mem_model_cb);
    tp->destroy_fn(tp->mem_model_cb);
    free(tp);

    printf("\n\n");
}


int main (void)
{
    ulongint        bus_latency = 7;
    ulongint        bus_width = 4;
    ulongint        tile_size = 128;

    Process_Method(10, bus_width, bus_latency, tile_size);

    return 0;
}


